@@ -2,7 +2,7 @@
  * pmt.c: PMT decoder/generator
  *----------------------------------------------------------------------------
  * (c)2001-2002 VideoLAN
- * $Id: pmt.c 99 2004-12-20 13:42:55Z massiot $
+ * $Id: pmt.c 108 2005-04-19 12:33:21Z gbazin $
  *
  * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
  *
@@ -112,6 +112,8 @@ void dvbpsi_DetachPMT(dvbpsi_handle h_dvbpsi)
   }
 
   free(h_dvbpsi->p_private_decoder);
+  if(h_dvbpsi->p_current_section)
+    dvbpsi_DeletePSISections(h_dvbpsi->p_current_section);
   free(h_dvbpsi);
 }
 
