@@ -2,7 +2,7 @@
  * sdt.c: SDT decoder/generator
  *----------------------------------------------------------------------------
  * (c)2001-2002 VideoLAN
- * $Id: sdt.c 99 2004-12-20 13:42:55Z massiot $
+ * $Id: sdt.c 110 2005-06-04 12:52:02Z gbazin $
  *
  * Authors: Johan Bilien <jobi@via.ecp.fr>
  *
@@ -88,6 +88,7 @@ int dvbpsi_AttachSDT(dvbpsi_decoder_t * p_psi_decoder, uint8_t i_table_id,
   p_subdec->pf_callback = &dvbpsi_GatherSDTSections;
   p_subdec->p_cb_data = p_sdt_decoder;
   p_subdec->i_id = (uint32_t)i_table_id << 16 | (uint32_t)i_extension;
+  p_subdec->pf_detach = dvbpsi_DetachSDT;
 
   /* Attach the subtable decoder to the demux */
   p_subdec->p_next = p_demux->p_first_subdec;
@@ -115,7 +116,7 @@ void dvbpsi_DetachSDT(dvbpsi_demux_t * p_demux, uint8_t i_table_id,
           uint16_t i_extension)
 {
   dvbpsi_demux_subdec_t* p_subdec;
-  dvbpsi_demux_subdec_t* p_next_subdec;
+  dvbpsi_demux_subdec_t** pp_prev_subdec;
   dvbpsi_sdt_decoder_t* p_sdt_decoder;
 
   unsigned int i;
@@ -131,8 +132,6 @@ void dvbpsi_DetachSDT(dvbpsi_demux_t * p_demux, uint8_t i_table_id,
     return;
   }
 
-  p_next_subdec = p_subdec->p_next;
-
   p_sdt_decoder = (dvbpsi_sdt_decoder_t*)p_subdec->p_cb_data;
 
   free(p_sdt_decoder->p_building_sdt);
@@ -140,12 +139,17 @@ void dvbpsi_DetachSDT(dvbpsi_demux_t * p_demux, uint8_t i_table_id,
   for(i = 0; i <= 255; i++)
   {
     if(p_sdt_decoder->ap_sections[i])
-      free(p_sdt_decoder->ap_sections[i]);
+      dvbpsi_DeletePSISections(p_sdt_decoder->ap_sections[i]);
   }
 
   free(p_subdec->p_cb_data);
+
+  pp_prev_subdec = &p_demux->p_first_subdec;
+  while(*pp_prev_subdec != p_subdec)
+    pp_prev_subdec = &(*pp_prev_subdec)->p_next;
+
+  *pp_prev_subdec = p_subdec->p_next;
   free(p_subdec);
-  p_subdec = p_next_subdec;
 }
 
 
@@ -462,7 +466,7 @@ void dvbpsi_DecodeSDTSections(dvbpsi_sdt_t* p_sdt,
   while(p_section)
   {
     for(p_byte = p_section->p_payload_start + 3;
-        p_byte < p_section->p_payload_end;)
+        p_byte + 4 < p_section->p_payload_end;)
     {
       uint16_t i_service_id = ((uint16_t)(p_byte[0]) << 8) | p_byte[1];
       int b_eit_schedule = (int)((p_byte[2] & 0x2) >> 1);
@@ -473,9 +477,12 @@ void dvbpsi_DecodeSDTSections(dvbpsi_sdt_t* p_sdt,
       dvbpsi_sdt_service_t* p_service = dvbpsi_SDTAddService(p_sdt,
           i_service_id, b_eit_schedule, b_eit_present,
           i_running_status, b_free_ca);
+
       /* Service descriptors */
       p_byte += 5;
       p_end = p_byte + i_length;
+      if( p_end > p_section->p_payload_end ) break;
+
       while(p_byte + 2 <= p_end)
       {
         uint8_t i_tag = p_byte[0];
